// Persistence of Vision Ray Tracer Scene Description File
// File: ?.pov
// Vers: 3.5
// Desc: Shows the use of trace() and Reorient_Trans() functions
// Date: mm/dd/yy
// Auth: ?
//

#version 3.5;

#include "colors.inc"
#include "transforms.inc"

global_settings {
  assumed_gamma 1.0
}
#declare Test = true;

// ----------------------------------------

camera {
  location  <0.0, 0.5, -80.0>
  direction 1.5*z
  right     x*image_width/image_height
  look_at   <0.0, 0.0,  0.0>
}

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  }
}

light_source {
  <0, 0, 0>            // light's position (translated below)
  color rgb <1, 1, 1>  // light's color
  translate <-30, 30, -30>
}

// ----------------------------------------

#include "LevellerUVrock_04.inc"
#declare TowerRock =
  object {LevellerRock_04
    scale <0.25,0.4,0.35>
    rotate 90*y
    //translate 40*y
    }
object {TowerRock}
// -------- random seed for the Bushes -------------
#declare Seed=seed(2);

// -------- Placing the bushes -------------

#declare Spacing = 3;
#declare Cnt = 0;


#declare PosX=-100;

#while (PosX < 100)

  #declare PosZ=-100;

  #while (PosZ < 100)

    // trace function
    #declare Norm = <0, 0, 0>;
    #declare Start = <PosX+(rand(Seed)-0.5)*Spacing, 10.0, PosZ+(rand(Seed)-0.5)*Spacing>;
    #declare Pos = trace (
                  TowerRock,     // object to test
                  Start,           // starting point
                  -y,              // direction
                  Norm );          // normal


    #if (Norm.x != 0 | Norm.y != 0 | Norm.z != 0)   // if intersection is found, normal differs from 0
      //#if ((vdot(Norm, y)>0.85) & (Pos.y < 100))
      // criteria for placing trees: not too steep and not too high
        object {
          //Bush1
          cylinder {
            0*y,  3*y,  0.5
            Reorient_Trans (y, Norm)
            pigment{color Red}
          }
          translate Pos
          //translate <30, -5, -120>
        }
        #declare Cnt=Cnt+1;
      //#end
    #end

    #declare PosZ=PosZ+Spacing;

  #end

  #declare PosX=PosX+Spacing;
#end

#debug concat("Placed ", str(Cnt,0,0), " Bushes on the rock\n")


